/** jQuery time field plugin
 *   jQuery.1.2.4 required
 * 
 * optional parameters: // $(elem).timefield( options ) 
 * * * * * * * * * * * * * * * * * * *
 * format - 12 || 24 - // default 24
 * display - 12 || 24 - // default = format
 * wheel - bool - enables mousewheel event // default = true // jquery.mousewheel.js should be included
 * drag - bool - enables click'n'drad increasing/decreasing // default = true
 * keys - bool - enables up'n'down keys for increasing/descreasing // default = true
 * hStep - int - hours increase and decrease step for mousewheel event // default = 1
 * mStep - int minutes increase and decrease step for mousewheel event // default = 1
 * dragStep - int - mousemove step // default = 5
 * allowEmpty - bool - if false or not set, current time will be used instead of empty value // default = false
 * autoCorrect - bool - enables auto correction // default = true
 * 
 * additional functions (public interface):
 * * * * * * * * * * * * * * * * * * *
 * timeSetFormat (format), timeSetDisplay(format), timeCheck(), timeMinutesStep(int), timeHoursStep(int),
 * timeToggleWheel([bool | on/off]), timeToggleDrag([bool | on/off]), timeToggleKeys([bool | on/off]), timeToggleAutoCorrect([bool | on/off])
 * 
 * notes
 * * * * * * * * * * * * * * * * * * *
 * if format and display are not the same, plugin adds hidden field with the same name attribute as time field's name
 * doubleclick as well as ctrl + arrow switches display format
 * ctrl + space as well as ctrl + click clears field ( sets current time if !allowEmpty)
 * tab and enter keys are not used (and shouldn't!)
 * 
 *  @author: johann kuindji (www.kuindji.com, www.stuffedguys.com, www.stuffedtracker.com)
 *  @email: jk@kuindji.com
 *  version 1.0
 *  
 *  example: 
 *  $('#input').timefield({format: 24, display: 12, mStep: 5});
 *  $('#input').timeSetDisplay(24);
 */

(function($){

$.fn.timefield = function( options ) {
    this.each( function() {
        var i = $(this);        
        if (this.tagName.toLowerCase() != 'input' || i.attr('type') != 'text') return false;
        if (!options) options = {};
        var th = this;
        this.tf = {
            format: (options.format || '24'), display: (options.display || options.format || 24),
            hStep: (parseInt(options.hStep) || 1), mStep: (parseInt(options.mStep) || 1),
            wheel: (typeof(options.wheel) != 'undefined' ? options.wheel : true),
            drag: (typeof(options.drag) != 'undefined' ? options.drag : true),
            keys: (typeof(options.keys) != 'undefined' ? options.keys : true),    
            hours: 0, minutes: 0,
            spelled: false, focus: false, 
            autoCorrect: (typeof(options.autoCorrect) != 'undefined' ? options.autoCorrect : false),
            allowEmpty: (typeof(options.allowEmpty) != 'undefined' ? options.allowEmpty : false),
            ofs: false, width: false, hidden: false,
            prevY: 0, dragStep: (parseInt(options.dragStep) || 5)
        };
        if ( this.tf.format != '12' && this.tf.format != '24') this.tf.format = 24;
        if ( this.tf.display != '12' && this.tf.display != '24') this.tf.display = 24;
        if ( this.tf.mStep < 1 ) this.tf.mStep = 1;
        if ( this.tf.hStep < 1 ) this.tf.hStep = 1;

        /* ************************************************************* */       

        this._timeValue = function( format ) { i.val( this[ format==12? '_format12':'_format24' ]() ); }
        this._format12 = function() { 
            if (this.tf.hours > 11 ) { var apm = 'pm'; var h = this.tf.hours - 12; }
            else { var apm = 'am'; var h = this.tf.hours; }
            return new String( h+':'+this._twoDigit(this.tf.minutes)+apm ); 
        }
        this._format24 = function() { return new String( this._twoDigit(this.tf.hours)+':'+this._twoDigit(this.tf.minutes) ); }    

        this._spell = function( str ) {
            if (!str.match(/\d{1,2}:\d{1,2}[apm]?/gi)) return false;
            var time = this._parseTime( str );
            if (time.apm) this.tf.hours = time.apm == 'pm' ? time.h + 12 : time.h;
            else this.tf.hours = time.h;
            this.tf.minutes = time.m;
            this._correctTime();
            return this.tf.spelled = true;
        }

        this._setCurrent = function() {
            var d = new Date;
            this.tf.hours = d.getHours();
            this.tf.minutes = d.getMinutes();
            if (this.tf.mStep > 1 || this.tf.hStep > 1) this._correctTime();
            this._timeValue( this.tf.display );
            this.tf.spelled = true;
        }

        this._parseTime = function( str ) {
            var time = str.split(':');
            if (typeof(time[1])=='undefined') time[1]='';
            time[2] = time[1].replace(/\d/g, '');
            time[1] = time[1].replace(/[amp]/gi, '');
            return {h: parseInt(time[0]), m: parseInt(time[1]), apm: time[2]};
        }

        this._check = function() {
            if (th.tf.spelled) return true;
            if (th._spell( i.val() ) ) return th._timeValue( th.tf.display );
            if (!th.tf.allowEmpty) return th._setCurrent(); 
            else i.val('');
            return true; 
        }

        this._ontimedrag = function( event ) {
            if ( Math.abs( event.clientY - th.tf.prevY ) < th.tf.dragStep ) return false;
            th._ontimewheel( event, (event.clientY > th.tf.prevY ? -1 : 1)  );
            th.tf.prevY = event.clientY;
            return false; 
        }

        this._ontimekeydown = function( event ) {
            var code = event.keyCode;
            var mode = $.inArray(code, [37,39]) > -1 ? 'h' : 'm';
            var delta = mode=='m' ? ( code == 38 ? 1 : -1) : ( code == 37 ? -1 : 1 ); 
            th._wheelTime( mode , delta );
        }

        this._ontimewheel = function( event, delta ) {
            if (!th.tf.ofs) th.tf.ofs = i.offset();
            if (!th.tf.width) th.tf.width = i.width();
            var mode = event.clientX - th.tf.ofs.left > th.tf.width / 2 ? 'm' : 'h';
            th._wheelTime( mode , delta );
            return true; 
        }

        this._wheelTime = function( mode, delta ) {
            if (mode == 'h') this.tf.hours += delta > 0 ? this.tf.hStep : -this.tf.hStep;
            if (mode == 'm') this.tf.minutes += delta > 0 ? this.tf.mStep : -this.tf.mStep;
            this._correctTime();
            this._timeValue( this.tf.display );
        }

        this._correctTime = function() {
            var remainder = 0;
            if (this.tf.mStep > 1 && (remainder = this.tf.minutes % this.tf.mStep) > 0) {
                this.tf.minutes += remainder < this.tf.mStep/2 ? this.tf.mStep-remainder : remainder-this.tf.mStep;
            }
            if (this.tf.minutes > 59) { this.tf.minutes = 0; this.tf.hours+=this.tf.hStep; } 
            if (this.tf.minutes < 0 ) { this.tf.minutes = 60-this.tf.mStep; this.tf.hours-=this.tf.hStep; }
    
            if (this.tf.hStep > 1 && (remainder = this.tf.hours % this.tf.hStep) > 0) {
                this.tf.hours += remainder < this.tf.hStep/2 ? this.tf.hStep-remainder : remainder-this.tf.hStep;
            }
            this.tf.hours = this.tf.hours > 23 ? 0 : this.tf.hours < 0 ? 24-this.tf.hStep : this.tf.hours;
        }

        this._twoDigit = function( str ) {
            str = new String(str);
            return str.length < 2 ? '0'+str : str; 
        }

        this._clear = function() {
            i.val(''); 
            this.tf.spelled = false;
            this._check();
            return false;
        }

        this._toggleParam = function(param, mode) {
            if (mode && mode != true && mode != false && mode.toLowerCase() != 'on' && mode.toLowerCase() != 'off') delete(mode);
            if (mode && typeof(mode)=='string') mode = mode.toLowerCase()=='on' ? true: false;
            var prevVal = this.tf[param]; 
            this.tf[param] = typeof(mode)!='undefined' ? mode : !this.tf[param];
            if (prevVal != this.tf[param]) {
                if ( param == 'wheel') this._toggleWheel( this.tf.wheel );
                if ( param == 'drag') this._toggleDrag( this.tf.wheel );
                if ( param == 'keys') this._toggleKeys( this.tf.wheel ); 
            }
        }

        /* ************************************************************* */       

        i.bind('keyup.time', function( event ) {
            if (!event.ctrlKey) i.unbind('click.time', this._clear);
            if (!this.tf.autoCorrect) return !this.tf.keys;
            var time = i.val().replace(/\s/g, '');
            time = time.replace(/[^0-9apm:]/gi, '');
            this.tf.spelled = false;
            i.val( time );
            return !this.tf.keys;
        });
        i.bind('blur.time click.time', function() {
            if (!this.tf.autoCorrect) return !this.tf.keys;
            this._check();
            return !this.tf.keys;
        });
        i.bind('blur.time', function() { this.tf.focus = false; return true; });
        i.bind('focus.time', function() { this.tf.focus = true; return true; });
        i.bind('dblclick', function(){ i.timeSetDisplay( (this.tf.display==12 ? 24:12) ); return false; });

        this._toggleWheel = function( on ) {
            if (on) {
                i.bind('mousewheel.time', function( event, delta ) {
                    if (!th.tf.focus) { th._ontimewheel( event, delta ); return false; }
                    else return true;
                });
                $('body').bind('mousewheel.time', function( event, delta ) {
                    if (th.tf.focus) { th._ontimewheel( event, delta ); return false; }
                    else return true;
                });
            } 
            else { i.unbind('mousewheel.time'); $('body').unbind('mousewheel.time'); }
        }

        this._toggleDrag = function( on ) {
            if ( on ) i.bind('mousedown.time', function( event ) {
                i.blur();
                $('body').bind('mouseup.time', function() { $('body').unbind('mousemove.time').unbind('mouseup.time'); i.focus(); return false;  });
                th.tf.prevY = event.clientY;
                $('body').bind('mousemove.time', th._ontimedrag);
                return false; 
            });
            else {
                i.unbind('mousedown.time');
                $('body').unbind('mousemove.time').unbind('mouseup.time'); 
            }
        }

        this._toggleKeys = function( on ) {
            if ( on ) i.bind('keydown.time', function( event ) {
                if (event.ctrlKey && event.keyCode == 32) return this._clear();
                if (event.ctrlKey) i.bind('click.time', this._clear);
                if ($.inArray( event.keyCode, [37,38,39,40] ) == -1) return true;
                if (event.ctrlKey) { i.dblclick(); return false; }
                th._ontimekeydown( event );
                return false;
            });
            else { i.unbind('keydown.time'); }
        }

        this._toggleWindow = function( on ) {
            if ( on ) $(window).bind('resize.time', function() {
                th.tf.ofs = i.offset();
                th.tf.width = i.width();
                return true;
            });
            else $(window).unbind('resize.time');
        }

        this._toggleForm = function( on ) {
            if ( on ) {
                this.tf.hidden = $('<input type=hidden>');
                this.tf.hidden.attr('name', i.attr('name'));
                i.after(this.tf.hidden);
                i.parents('form:first').bind('submit.time', function(){ 
                    th.tf.hidden.val( th[ th.tf.format==12? '_format12':'_format24' ]() ); 
                });
            }
            else {
                i.parents('form:first').unbind('submit.time');
                this.tf.hidden.remove();
                this.tf.hidden = false;
            }
        }

        if (this.tf.wheel) this._toggleWheel( true );
        if (this.tf.drag) this._toggleDrag( true ); 
        if (this.tf.keys) this._toggleKeys( true );
        if (this.tf.wheel || this.tf.drag) this._toggleWindow( true );
        if (this.tf.format != this.tf.display) this._toggleForm( true );

        /* ************************************************************* */       

        if (!i.val() && !this.tf.allowEmpty ) this._setCurrent();
        else this._check();
    })
}

$.fn.timeSetFormat = function( format ) {
    if (!format || (format != 12 && format != 24) ) format = 24;
    this.each( function() { 
        this.tf.format = format;
        $(this).timeSetDisplay( this.tf.display );     
    });
}
$.fn.timeSetDisplay = function( format ) {
    if (format != 12 && format != 24 ) format = false;
    this.each( function() {
        var prevDisplay = this.tf.display; 
        this.tf.display = format ? format : this.tf.format;
        if (prevDisplay != this.tf.display) { this._toggleForm( (this.tf.display != this.tf.format) ); this._timeValue( this.tf.display ); }
    });
}
$.fn.timeMinutesStep = function( step ) { this.each(function() {
    var prevStep = this.tf.mStep;
    this.tf.mStep = parseInt(step) > 0 ? parseInt(step) : this.tf.mStep;
    if (prevStep != this.tf.mStep) { this._correctTime(); this._timeValue( this.tf.display );} 
});}
$.fn.timeHoursStep = function( step ) { this.each(function() {
    var prevStep = this.tf.hStep;
    this.tf.hStep = parseInt(step) > 0 ? parseInt(step) : this.tf.hStep;
    if (prevStep != this.tf.hStep) { this._correctTime(); this._timeValue( this.tf.display );} 
});}
$.fn.timeCheck = function() { this.each( function() { this._check(); }); }
$.fn.timeToggleWheel = function( mode ) { this.each( function() { this._toggleParam('wheel', mode);});}
$.fn.timeToggleDrag = function( mode ) { this.each( function() { this._toggleParam('drag', mode); });}
$.fn.timeToggleKeys = function( mode ) { this.each( function() { this._toggleParam('keys', mode); });}
$.fn.timeToggleAutoCorrect = function( mode ) { this.each( function() { 
    this._toggleParam('autoCorrect', mode);
    if (this.tf.autoCorrect) this._check(); 
});}

})(jQuery);